function [est, confint, p, f, SSH] = rscomparative(y, coord, d, A, alfa)

% RSCOMPARATIVE - analyse comparative experiments. Version 0.1.23.
%
% The syntax is
% [est, confint, p, f, SSH] = rscomparative(y, coord, [d], [A], [alpha])
%
% Arguments
% y is the vector of observations
% coord is a matrix of coordinates: which treatment, block, etc. is the corresponding observation from?
% [d] and [A] are voluntary arguments. If they are provided then the linear hypothesis A*est=d is tested.
% [alpha] determines the confidence level of the 1-alpha confidence intervals for the estimated parameters.
%
% rscomparative uses the model
% y = tau1*z1 + ... + beta1*v1 + ... + gamma1*w1+ ... + epsilon,
% with the additional condition that the sum of the betas, the sum of the gammas etc. all equal zero.
%
% Return values
% est is the least squares estimate
% confint are 1-alpha-confidence intervals for the estimated parameters
% p is the p value for the linear hypothesis test
% f is the test statistics
% SSH is the hypothesis sum of squares
% 
%
% References
% Neter, Wasserman, Kutner: Applied Linear Regression Models, R.D. Irwin Inc., 1983, Chapter 9.


% Copyright (C) 2016 Mikael Kurula <mkurula@abo.fi>
%
% This program is free software; you can redistribute it and/or modify it under
% the terms of the GNU General Public License as published by the Free Software
% Foundation; either version 3 of the License, or (at your option) any later
% version.
%
% This program is distributed in the hope that it will be useful, but WITHOUT
% ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
% FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
% details.
%
% You should have received a copy of the GNU General Public License along with
% this program; if not, see <http://www.gnu.org/licenses/>.
%
% Author: Mikael Kurula <mkurula@abo.fi>, 2016


% Check input
if nargin < 2 || nargin > 5
    error('Try help rscomparative for instructions.') ;
end
if size(y,2) > 1,
    error('y has more than one column. Try help rscomparative for instructions.');
end
if nargin < 5,
  alfa=0.05;
end

% Build data matrix and augment response vector
[N, m]=size(coord);
X=[];
for j=1:m,
  co=(coord(:,j));
  n(j)=max(co);
  Id=eye(n(j));
  Xj=Id(co,:);
  X=[X Xj];
end
Xj=[];
for j=2:m,
  for l=1:m,
  if j==l,
    Xj=[Xj ones(1,n(l))];
  else
    Xj=[Xj zeros(1,n(l))];
  end
  end
  X=[X ; Xj];
end  
y=[y ; zeros(m-1,1)];

% Do regression and calculate confidence intervals
est=X\y;
r=y-X*est;
SSE=r'*r;
dfs=n-1;
dfE=N-1-sum(dfs);
MSE=SSE/dfE;
C=inv(X'*X);           
d_i=sqrt(MSE*diag(C));
t_alfa_2=tinv(1-alfa/2,dfE);
confint=[est-d_i*t_alfa_2 est+d_i*t_alfa_2];

% Test linear hypothesis if asked for
if nargin > 3 && size(A,1)>0,
  [p, f, SSH] = rslinhyp(y, X, d, A); % KOLLA FRIHETSGRADER RÄTT!
else
  p=1; f=0; SSH=0;
end

end