function [cor] = rsresid(y, X, additional, prefix)
% RSRESID - Graphically check residual for model adequacy. Version 0.1.21.
%
% The residual is plotted against the normal distribution in a Q-Q plot.
% Scatter-plots are drawn for the residual against time, the response predicted by the model, and any additional variables requested by the user;
% the correlations between the residual and all of these varibles is calculated. (The correlation between the residual and the prediction vector is 0.)
% In a separate plot, the residual is plotted against time with 0.95-confidence intervals to aid in outlier detection.
%
% The syntax is
% [cor] = rsresid(y, X, [additional], [prefix])
%
% Arguments
% y is an N by 1 column vector of data to fit.
% X is an N by a matrix containing the values of a predictors used in the regression.
% [additional] is a voluntary parameter, a matrix whose columns are any variables against which one wants to calculate correlation and plot the residual. Enter the empty matrix [] here if you want files saved but no additional variables.
% [prefix] is a voluntary parameter, a string which causes rsresid to save the plots as pdf files. [prefix] should be a string which will be prepended to the file names of the pdf files.
%
% Return value, a column vector:
% cor are the correlation coefficients between the residual and time (1:N)' as well as any variables in [additional]. This vector has length 3+n, where n is the width of the optional matrix additional.
%
% References
% Draper & Smith: Applied Regression Analysis, 3rd ed., Wiley, 1998, Chapter 2.
% Neter, Wasserman & Kutner: Applied Linear Regression Models, Richard D. Irwin Inc., 1983, Chapter 4.
% Myers, Montgomery & Anderson-Cook: Response Surface Methodology, 3rd ed., Wiley, 2009, Section 2.7.


% Copyright (C) 2016 Mikael Kurula <mkurula@abo.fi>
%
% This program is free software; you can redistribute it and/or modify it under
% the terms of the GNU General Public License as published by the Free Software
% Foundation; either version 3 of the License, or (at your option) any later
% version.
%
% This program is distributed in the hope that it will be useful, but WITHOUT
% ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
% FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
% details.
%
% You should have received a copy of the GNU General Public License along with
% this program; if not, see <http://www.gnu.org/licenses/>.
%
% Author: Mikael Kurula <mkurula@abo.fi>, 2016


% Check input
if nargin < 2 || nargin > 4,
    error('Wrong number of arguments. Try help rsresid for instructions.') ;
end
if size(y,2) > 1,
    error('y has more than one column. Try help rsresid for instructions.');
end
if cond(X'*X) > 1e10,
    warning('The normal equation is ill-conditioned. If the matrix X does not have linearly dependent columns, try standardizing the factor data.');
end

% Take care of differences between Octave and Matlab
if exist ('OCTAVE_VERSION', 'builtin')       % Are we running Octave?
   graphics_toolkit('gnuplot')
   PprSz=[6 4];
else
   PprSz=[12 8];
end 

% Standard regression
[N,a]=size(X);
[b,~,r,rint]=regress(y,X);
yhat=X*b;

% Check that the residual is normally distributed
figure('PaperPosition',[0 0 PprSz],'PaperSize',PprSz);
normplot(r)
if nargin == 4 && size(prefix,1)>0,
   print([prefix,'-rsresid-normplot'],'-dpdf')
end

% Calculate correlation and draw scatter plots
Time=(1:N)';
cor(1,1)=corr(r,Time);

figure('PaperPosition',[0 0 PprSz],'PaperSize',PprSz);
plot(Time,r,'+','linewidth',2)
title('Residual vs. time')
xlabel('Time 1:N (table order of X)')
ylabel('Residual')
xlim([0 N+1]);
if nargin == 4 && size(prefix,1)>0,
   print([prefix,'-rsresid-time'],'-dpdf')
end

figure('PaperPosition',[0 0 PprSz],'PaperSize',PprSz);
plot(yhat,r,'+','linewidth',2)
title('Residual vs. predicted response')
xlabel('Predicted response y-hat')
ylabel('Residual')

xvar=yhat;
if min(xvar)>0
  Lo=0.95*min(xvar);
elseif min(xvar)<0
  Lo=1.05*min(xvar);
else
  Lo=-0.05;
end
if max(xvar)<0
  Up=0.95*max(xvar);
elseif max(xvar)>0
  Up=1.05*max(xvar);
else
  Up=0.05;
end
xlim([Lo Up]);

if nargin == 4 && size(prefix,1)>0,
   print([prefix,'-rsresid-pred'],'-dpdf')
end

% Do the same for additional predictors, if given
if nargin>2,
  [~, n]=size(additional); 
  for i=1:n,
    vari=additional(:,i);
    cor(1+i,1)=corr(r,vari);
    figure('PaperPosition',[0 0 PprSz],'PaperSize',PprSz);
    plot(vari,r,'+','linewidth',2)
    title(['Residual vs. additional predictor #',int2str(i)])
    xlabel(['Additional predictor #',int2str(i)])
    ylabel('Residual')

    xvar=vari;
    if min(xvar)>0
      Lo=0.95*min(xvar);
    elseif min(xvar)<0
      Lo=1.05*min(xvar);
    else
      Lo=-0.05;
    end
    if max(xvar)<0
      Up=0.95*max(xvar);
    elseif max(xvar)>0
      Up=1.05*max(xvar);
    else
      Up=0.05;
    end
    xlim([Lo Up]);

    if nargin == 4 && size(prefix,1)>0,
      print([prefix,'-rsresid-add',int2str(i)],'-dpdf')
    end
  end
end

%Finally, plot residual with confidence intervals
figure('PaperPosition',[0 0 PprSz],'PaperSize',PprSz);
h=errorbar(r, (rint(:,2)-rint(:,1))/2,'+')
set(h,'linewidth',2)
hold on
plot([0 N+1],[0 0],'r','linewidth',2)
hold off
title('Residual with 0.95-confidence intervals')
xlabel('Time 1:N (table order of X)')
ylabel('Residual')
xlim([0 N+1]);
if nargin == 4 && size(prefix,1)>0,
   print([prefix,'-rsresid-confint'],'-dpdf')
end

end
